package org.jenkinsci.plugins.p4_client.workspace;

import hudson.Extension;
import hudson.model.AutoCompletionCandidates;
import hudson.util.FormValidation;

import java.util.List;

import org.jenkinsci.plugins.p4_client.client.ConnectionFactory;
import org.kohsuke.stapler.DataBoundConstructor;
import org.kohsuke.stapler.QueryParameter;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.client.IClientSummary;
import com.perforce.p4java.server.IOptionsServer;

public class StaticWorkspaceImpl extends Workspace {

	private final String charset;
	private final String name;

	@Override
	public String getCharset() {
		return charset;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public WorkspaceType getType() {
		return WorkspaceType.STATIC;
	}

	@DataBoundConstructor
	public StaticWorkspaceImpl(String charset, String name) {
		this.charset = charset;
		this.name = name;
	}

	@Override
	public IClient setClient(IOptionsServer connection, String user)
			throws Exception {
		IClient iclient = connection.getClient(getName());
		if (iclient == null) {
			throw new Exception("Undefined workspace: " + getName());
		}
		return iclient;
	}

	@Extension
	public static final class DescriptorImpl extends WorkspaceDescriptor {

		@Override
		public String getDisplayName() {
			return "Static (static view, master only)";
		}

		/**
		 * Provides auto-completion for workspace names. Stapler finds this
		 * method via the naming convention.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteName(
				@QueryParameter String value) {

			AutoCompletionCandidates c = new AutoCompletionCandidates();
			try {
				IOptionsServer iserver = ConnectionFactory.getConnection();
				if (iserver != null && value.length() > 0) {
					String user = iserver.getUserName();
					List<IClientSummary> list;
					list = iserver.getClients(user, value + "*", 10);
					for (IClientSummary l : list) {
						c.add(l.getName());
					}
				}
			} catch (Exception e) {
			}

			return c;
		}

		public FormValidation doCheckName(@QueryParameter String value) {
			try {
				IOptionsServer p4 = ConnectionFactory.getConnection();
				IClient client = p4.getClient(value);
				if (client != null) {
					return FormValidation.ok();
				}
				return FormValidation.error("Unknown Client: " + value);
			} catch (Exception e) {
				return FormValidation.error("Perforce connection error.");
			}
		}

	}
}
